export { Result, T, Err, ok, err, isOk, isErr, map };
/**
 * A `Result` is either an `Ok` containing data of type `V`, or an `Err` object
 * containing an error value of type `E`.
 */
declare type Result<V, E> = V | Err<E>;
/** Alias for the `Result` type. */
declare type T<V, E> = Result<V, E>;
/**
 * The `Err` class extends the built in JS `Error` class. This means that
 * `Err`s have the same behavior as `Error`s when thrown. We add a `value`
 * field to hold the data associated with the error.
 */
declare class Err<E> extends Error {
    value: E;
    constructor(value: E);
}
/**
 * Create a wrapped type where each member of `T` is a `Result` with error
 * value `E`.
 */
declare type ResultMapped<T, E> = {
    [k in keyof T]: Result<T[k], E>;
};
/**
 * A constructor for the `Ok` variant of `Result`, which is effectively the
 * identity function.
 */
declare const ok: <V, E>(v: V) => Result<V, E>;
/** A constructor for the `Err` variant of `Result`. */
declare const err: <V, E>(e: E) => Result<V, E>;
/** Returns true and narrows type if `x` is not an `Err`. */
declare const isOk: <V, E>(x: Result<V, E>) => x is V;
/** Returns true and narrows type if `x` is an `Err`. */
declare const isErr: <V, E>(x: Result<V, E>) => x is Err<E>;
/** Apply `fn` if none of `resultArgs` are `Err`s. Otherwise return the first `Err`. */
declare const map: <Args extends any[], R, E>(fn: (...args: Args) => Result<R, E>, ...resultArgs: ResultMapped<Args, E>) => Result<R, E>;
